@@ -39,7 +39,7 @@ from pysollib.stack import \
         StackWrapper, \
         WasteStack, \
         WasteTalonStack
-from pysollib.util import ACE, ANY_RANK, ANY_SUIT, JACK, KING, NO_RANK, \
+from pysollib.util import ACE, ANY_RANK, ANY_SUIT, JACK, KING, \
         QUEEN, RANKS, \
         UNLIMITED_CARDS
 # ************************************************************************
@@ -105,6 +105,11 @@ class Camelot_RowStack(ReserveStack):
             self.playMoveMove(1, game.s.foundations[0], sound=False)
             self.fillStack()
             return True
+        if not self.cards and game.s.talon.cards and \
+                self.acceptsCards(game.s.talon, [game.s.talon.cards[-1]]):
+            game.s.talon.playMoveMove(1, self)
+            return True
+
         return False
 
     def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
@@ -164,10 +169,26 @@ class Camelot(Game):
         w = l.XS
         self.setSize(l.XM + w + 4*l.XS + w + l.XS, l.YM + 4*l.YS)
         # create stacks
+        stackNum = 0
+        font = self.app.getFont("canvas_default")
         for i in range(4):
             for j in range(4):
                 x, y = l.XM + w + j*l.XS, l.YM + i*l.YS
-                s.rows.append(self.RowStack_Class(x, y, self))
+                stack = self.RowStack_Class(x, y, self)
+                if self.preview <= 1:
+                    stack.texts.misc = MfxCanvasText(self.canvas,
+                                                     x + l.CW // 2,
+                                                     y + l.CH // 2,
+                                                     anchor="center",
+                                                     font=font)
+                    if stackNum in (0, 3, 12, 15):
+                        stack.texts.misc.config(text="K")
+                    elif stackNum in (1, 2, 13, 14):
+                        stack.texts.misc.config(text="Q")
+                    elif stackNum in (4, 7, 8, 11):
+                        stack.texts.misc.config(text="J")
+                s.rows.append(stack)
+                stackNum += 1
         x, y = l.XM, l.YM
         s.talon = self.Talon_Class(x, y, self)
         l.createText(s.talon, 's')
@@ -195,6 +216,8 @@ class Camelot(Game):
         return len(self.s.talon.cards) == 0
 
     def isRowsFill(self):
+        if len(self.s.talon.cards) == 0:
+            return True
         for i in range(16):
             if len(self.s.rows[i].cards) == 0:
                 return False
@@ -582,74 +605,10 @@ class GrandmammasPatience(Game):
         p.dump(self.base_rank)
 
 
-# ************************************************************************
-# * Double Line
-# ************************************************************************
-
-class DoubleLine_RowStack(BasicRowStack):
-    def acceptsCards(self, from_stack, cards):
-        if not BasicRowStack.acceptsCards(self, from_stack, cards):
-            return False
-        # this stack accepts any one card from the Waste pile
-        return from_stack is self.game.s.waste
-
-    def getHelp(self):
-        return _('Tableau. Build regardless of rank and suit.')
-
-
-class DoubleLine(Game):
-
-    def createGame(self):
-
-        l, s = Layout(self), self.s
-        h0 = l.YS+3*l.YOFFSET
-        self.setSize(l.XM+10*l.XS, l.YM+l.YS+l.TEXT_HEIGHT+2*h0)
-
-        x, y = l.XM, l.YM
-        s.talon = WasteTalonStack(x, y, self, max_rounds=1)
-        l.createText(s.talon, 's')
-        x += l.XS
-        s.waste = WasteStack(x, y, self)
-        l.createText(s.waste, 's')
-
-        x += l.XS
-        for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=i % 4))
-            x += l.XS
-        for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=i,
-                                 base_rank=KING, dir=-1))
-            x += l.XS
-
-        y = l.YM+l.YS+l.TEXT_HEIGHT
-        for i in range(2):
-            x = l.XM
-            for j in range(10):
-                s.rows.append(DoubleLine_RowStack(x, y, self, max_cards=2,
-                              max_move=1, max_accept=1, base_rank=NO_RANK))
-                x += l.XS
-            y += h0
-
-        l.defaultStackGroups()
-
-    def startGame(self):
-        self.startDealSample()
-        self.s.talon.dealRow()
-        self.s.talon.dealCards()
-
-    def fillStack(self, stack):
-        if stack in self.s.rows and not stack.cards:
-            old_state = self.enterState(self.S_FILL)
-            if not self.s.waste.cards:
-                self.s.talon.dealCards()
-            if self.s.waste.cards:
-                self.s.waste.moveMove(1, stack)
-            self.leaveState(old_state)
-
-
 # register the game
 registerGame(GameInfo(280, Camelot, "Camelot",
-                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_BALANCED))
+                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_BALANCED,
+                      altnames=("Kings in the Corners")))
 registerGame(GameInfo(610, SlyFox, "Sly Fox",
                       GI.GT_NUMERICA, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(614, OpenSlyFox, "Open Sly Fox",
@@ -659,5 +618,3 @@ registerGame(GameInfo(623, PrincessPatience, "Princess Patience",
                       GI.GT_2DECK_TYPE, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(622, GrandmammasPatience, "Grandmamma's Patience",
                       GI.GT_NUMERICA, 2, 0, GI.SL_MOSTLY_SKILL))
-registerGame(GameInfo(702, DoubleLine, "Double Line",
-                      GI.GT_NUMERICA, 2, 0, GI.SL_BALANCED))
