@@ -211,11 +211,11 @@ class Mahjongg_RowStack(OpenStack):
         elif TOOLKIT == 'kivy':
             rows = [s for s in self.game.s.rows[:self.id] if s.cards]
             if rows:
-                # self.group.tkraise(rows[-1].group)
+                self.group.tkraise(rows[-1].group)
                 return
             rows = [s for s in self.game.s.rows[self.id+1:] if s.cards]
             if rows:
-                # self.group.lower(rows[0].group)
+                self.group.lower(rows[0].group)
                 return
         elif TOOLKIT == 'gtk':
             # FIXME (this is very slow)
@@ -383,7 +383,7 @@ class AbstractMahjonggGame(Game):
         # dx, dy = 2, -2
         # dx, dy = 3, -3
         cs = self.app.images.cs
-        if cs.version >= 6:
+        if cs.version == 6 or cs.mahjongg3d:
             dx = l.XOFFSET
             dy = -l.YOFFSET
             d_x = cs.SHADOW_XOFFSET
