@@ -67,7 +67,8 @@ class StatsFormatter(PysolStatsFormatter):
                            6, result[6],
                            7, result[7])
         total, played, won, lost, time, moves, perc = self.getStatSummary()
-        text = _("Total (%d out of %d games)") % (played, total)
+        text = _("Total (%(played)d out of %(total)d games)") % {
+            'played': played, 'total': total}
         iter = self.store.append(None)
         self.store.set(iter,
                        0, text,
@@ -526,7 +527,8 @@ class Status_StatsDialog(MfxMessageDialog):  # MfxDialog
         w1 = (_('Highlight piles: ') + str(stats.highlight_piles) + '\n' +
               _('Highlight cards: ') + str(stats.highlight_cards) + '\n' +
               _('Highlight same rank: ') +
-              str(stats.highlight_samerank) + '\n')
+              str(stats.highlight_samerank) + '\n' +
+              _('Face-down peeks: ') + str(stats.peeks) + '\n')
         if game.s.talon:
             if game.gameinfo.redeals != 0:
                 w2 += _('\nRedeals: ') + str(game.s.talon.round - 1)
