@@ -33,8 +33,8 @@ from pysollib.ui.tktile.tkutil import bind, loadImage
 
 from six.moves import tkinter
 from six.moves import tkinter_font
+from six.moves import tkinter_ttk as ttk
 
-from . import ttk
 from .tkwidget import MfxDialog, MfxMessageDialog
 
 
@@ -368,7 +368,8 @@ class TreeFormatter(PysolStatsFormatter):
             self.parent_window.games[id] = t8
 
         total, played, won, lost, time_, moves, perc = self.getStatSummary()
-        text = _("Total (%d out of %d games)") % (played, total)
+        text = _("Total (%(played)d out of %(total)d games)") % {
+            'played': played, 'total': total}
         id = self.tree.insert("", "end", text=text,
                               values=(won+lost, won, lost, time_, moves, perc))
         self.parent_window.tree_items.append(id)
@@ -479,7 +480,8 @@ class AllGamesFrame(ttk.Frame):
 
     def fillTreeview(self, player):
         if self.tree_items:
-            self.tree.delete(tuple(self.tree_items))
+            for item in self.tree.get_children():
+                self.tree.delete(item)
             self.tree_items = []
         self.formatter.writeStats(player, sort_by=self.sort_by)
         if self.dialog.buttons:
@@ -613,7 +615,8 @@ class Status_StatsDialog(MfxMessageDialog):
         w1 = (_("Highlight piles: ") + str(stats.highlight_piles) + "\n" +
               _("Highlight cards: ") + str(stats.highlight_cards) + "\n" +
               _("Highlight same rank: ") +
-              str(stats.highlight_samerank) + "\n")
+              str(stats.highlight_samerank) + "\n" +
+              _('Face-down peeks: ') + str(stats.peeks) + '\n')
         if game.s.talon:
             if game.gameinfo.redeals != 0:
                 w2 = w2 + _("\nRedeals: ") + str(game.s.talon.round - 1)
@@ -808,8 +811,8 @@ class TopFrame(ttk.Frame):
         #                 s.score_casino_result.min,
         #                 s.score_casino_result.max,
         #                 round(s.score_casino_result.average, 2), ))
-        for l, min, max, avr, tot, top in ll:
-            ttk.Label(frame, text=l
+        for label, min, max, avr, tot, top in ll:
+            ttk.Label(frame, text=label
                       ).grid(row=row, column=0, padx=5, pady=5)
             ttk.Label(frame, text=str(min)
                       ).grid(row=row, column=1, padx=5, pady=5)
@@ -857,7 +860,7 @@ class ProgressionFrame(ttk.Frame):
         w = dialog.tkfont.measure('M') * 42
         w = max(w, 500)
         w = min(w, 600)
-        self.canvas_width, self.canvas_height = w, 250
+        self.canvas_width, self.canvas_height = w, 325
         cond = parent.winfo_screenwidth() < 800 or \
             parent.winfo_screenheight() < 600
         if cond:
@@ -888,8 +891,9 @@ class ProgressionFrame(ttk.Frame):
                             variable=var, value='current',
                             command=self.updateGraph)
         b.pack(fill='x', expand=True, padx=3, pady=1)
+
         label_frame = ttk.LabelFrame(right_frame, text=_('Statistics for'))
-        label_frame.pack(side='top', fill='x', pady=10)
+        label_frame.pack(side='top', fill='x', pady=5)
         self.variable = var = tkinter.StringVar()
         var.set('week')
         for v, t in (
@@ -901,8 +905,9 @@ class ProgressionFrame(ttk.Frame):
             b = ttk.Radiobutton(label_frame, text=t, variable=var,
                                 value=v, command=self.updateGraph)
             b.pack(fill='x', expand=True, padx=3, pady=1)
+
         label_frame = ttk.LabelFrame(right_frame, text=_('Show graphs'))
-        label_frame.pack(side='top', fill='x')
+        label_frame.pack(side='top', fill='x', pady=5)
         self.played_graph_var = tkinter.BooleanVar()
         self.played_graph_var.set(True)
         b = ttk.Checkbutton(label_frame, text=_('Played'),
@@ -922,6 +927,21 @@ class ProgressionFrame(ttk.Frame):
                             variable=self.percent_graph_var)
         b.pack(fill='x', expand=True, padx=3, pady=1)
 
+        label_frame = ttk.LabelFrame(right_frame, text=_('Date format'))
+        label_frame.pack(side='top', fill='x', pady=5)
+        self.date_format = tkinter.StringVar()
+        self.date_format.set(self.app.opt.date_format)
+        b = ttk.Radiobutton(label_frame, text=_('MM-DD'),
+                            command=self.updateDateFormat,
+                            variable=self.date_format,
+                            value='%m-%d')
+        b.pack(fill='x', expand=True, padx=3, pady=1)
+        b = ttk.Radiobutton(label_frame, text=_('DD-MM'),
+                            command=self.updateDateFormat,
+                            variable=self.date_format,
+                            value='%d-%m')
+        b.pack(fill='x', expand=True, padx=3, pady=1)
+
         # self.createGraph()
         bind(canvas, '<Map>', self.createGraph)
 
@@ -988,6 +1008,10 @@ class ProgressionFrame(ttk.Frame):
 
         self.updateGraph()
 
+    def updateDateFormat(self, *args):
+        self.app.opt.date_format = self.date_format.get()
+        self.updateGraph()
+
     def updateGraph(self, *args):
         interval = self.variable.get()
         canvas = self.canvas
@@ -996,14 +1020,20 @@ class ProgressionFrame(ttk.Frame):
         self.items = []
 
         all_games = (self.all_games_variable.get() == 'all')
-        result = self.formatter.getResults(interval, all_games)
+
+        date_format = self.date_format.get()
 
         if interval in ('week', 'month'):
             t = _('Games/day')
         else:
             t = _('Games/week')
+            date_format += '-%y'
+
         canvas.itemconfig(self.games_text_id, text=t)
 
+        result = self.formatter.getResults(interval, all_games,
+                                           date_format)
+
         graph_width = self.canvas_width-self.left_margin-self.right_margin
         graph_height = self.canvas_height-self.top_margin-self.bottom_margin
         dx = (graph_width-2*self.graph_dx)//(len(result)-1)
