@@ -1,50 +1,48 @@
 #!/usr/bin/env python
 # -*- mode: python; coding: utf-8; -*-
-##---------------------------------------------------------------------------##
-##
-## Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
-## Copyright (C) 2003 Mt. Hood Playing Card Co.
-## Copyright (C) 2005-2009 Skomoroh
-##
-## This program is free software: you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation, either version 3 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
-##
-##---------------------------------------------------------------------------##
+# ---------------------------------------------------------------------------
+#
+# Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
+# Copyright (C) 2003 Mt. Hood Playing Card Co.
+# Copyright (C) 2005-2009 Skomoroh
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# ---------------------------------------------------------------------------
 
-
-# imports
 import os
-import Tkinter
-from UserList import UserList
 
-# PySol imports
-from pysollib.mfxutil import destruct, Struct, KwStruct
-from pysollib.mfxutil import format_time
 from pysollib.gamedb import GI
-from pysollib.help import help_html
+from pysollib.mfxutil import KwStruct, Struct, destruct
+from pysollib.mfxutil import format_time
+from pysollib.mygettext import _
 from pysollib.resource import CSI
+from pysollib.ui.tktile.selecttree import SelectDialogTreeData
+from pysollib.ui.tktile.tkutil import unbind_destroy
 
-# Toolkit imports
-from tkutil import unbind_destroy
-from tkwidget import MfxDialog, MfxScrolledCanvas
-from selecttree import SelectDialogTreeLeaf, SelectDialogTreeNode
-from selecttree import SelectDialogTreeData, SelectDialogTreeCanvas
+from six.moves import UserList
+from six.moves import tkinter
 
+from .selecttree import SelectDialogTreeCanvas
+from .selecttree import SelectDialogTreeLeaf, SelectDialogTreeNode
+from .tkwidget import MfxDialog, MfxScrolledCanvas
 
 # ************************************************************************
 # * Nodes
 # ************************************************************************
 
+
 class SelectGameLeaf(SelectDialogTreeLeaf):
     pass
 
@@ -62,7 +60,7 @@ class SelectGameNode(SelectDialogTreeNode):
             for gi in self.tree.data.all_games_gi:
                 if gi and self.select_func is None:
                     # All games
-                    ##name = '%s (%s)' % (gi.name, CSI.TYPE_NAME[gi.category])
+                    # name = '%s (%s)' % (gi.name, CSI.TYPE_NAME[gi.category])
                     name = gi.name
                     node = SelectGameLeaf(self.tree, self, name, key=gi.id)
                     contents.append(node)
@@ -80,10 +78,13 @@ class SelectGameNode(SelectDialogTreeNode):
 class SelectGameData(SelectDialogTreeData):
     def __init__(self, app):
         SelectDialogTreeData.__init__(self)
-        self.all_games_gi = map(app.gdb.get, app.gdb.getGamesIdSortedByName())
-        self.no_games = [ SelectGameLeaf(None, None, _("(no games)"), None), ]
+        self.all_games_gi = list(map(
+            app.gdb.get,
+            app.gdb.getGamesIdSortedByName()))
+        self.no_games = [SelectGameLeaf(None, None, _("(no games)"), None), ]
         #
-        s_by_type = s_oriental = s_special = s_original = s_contrib = s_mahjongg = None
+        s_by_type = s_oriental = s_special = s_original = s_contrib = \
+            s_mahjongg = None
         g = []
         for data in (GI.SELECT_GAME_BY_TYPE,
                      GI.SELECT_ORIENTAL_GAME_BY_TYPE,
@@ -93,18 +94,22 @@ class SelectGameData(SelectDialogTreeData):
                      ):
             gg = []
             for name, select_func in data:
-                if name is None or not filter(select_func, self.all_games_gi):
+                if name is None or not list(filter(
+                        select_func, self.all_games_gi)):
                     continue
                 gg.append(SelectGameNode(None, _(name), select_func))
             g.append(gg)
-        select_mahjongg_game = lambda gi: gi.si.game_type == GI.GT_MAHJONGG
+
+        def select_mahjongg_game(gi):
+            return gi.si.game_type == GI.GT_MAHJONGG
+
         gg = None
-        if filter(select_mahjongg_game, self.all_games_gi):
+        if list(filter(select_mahjongg_game, self.all_games_gi)):
             gg = SelectGameNode(None, _("Mahjongg Games"),
                                 select_mahjongg_game)
         g.append(gg)
         if g[0]:
-            s_by_type = SelectGameNode(None, _("French games"),
+            s_by_type = SelectGameNode(None, _("French Games"),
                                        tuple(g[0]), expanded=1)
         if g[1]:
             s_oriental = SelectGameNode(None, _("Oriental Games"),
@@ -115,55 +120,61 @@ class SelectGameData(SelectDialogTreeData):
         if g[3]:
             s_original = SelectGameNode(None, _("Original Games"),
                                         tuple(g[3]))
-##         if g[4]:
-##             s_contrib = SelectGameNode(None, "Contributed Games", tuple(g[4]))
+        # if g[4]:
+        #   s_contrib = SelectGameNode(None, "Contributed Games", tuple(g[4]))
         if g[5]:
             s_mahjongg = g[5]
         #
         s_by_compatibility, gg = None, []
         for name, games in GI.GAMES_BY_COMPATIBILITY:
-            select_func = lambda gi, games=games: gi.id in games
-            if name is None or not filter(select_func, self.all_games_gi):
+            def select_func(gi, games=games):
+                return gi.id in games
+            if name is None or not list(filter(
+                    select_func, self.all_games_gi)):
                 continue
             gg.append(SelectGameNode(None, name, select_func))
         if 1 and gg:
             s_by_compatibility = SelectGameNode(None, _("by Compatibility"),
                                                 tuple(gg))
-            pass
         #
         s_by_pysol_version, gg = None, []
         for name, games in GI.GAMES_BY_PYSOL_VERSION:
-            select_func = lambda gi, games=games: gi.id in games
-            if name is None or not filter(select_func, self.all_games_gi):
-               continue
-            name = _("New games in v. ") + name
+            def select_func(gi, games=games):
+                return gi.id in games
+            if name is None or not list(filter(
+                    select_func, self.all_games_gi)):
+                continue
+            name = _("New games in v. %(version)s") % {'version': name}
             gg.append(SelectGameNode(None, name, select_func))
         if 1 and gg:
             s_by_pysol_version = SelectGameNode(None, _("by PySol version"),
                                                 tuple(gg))
         s_by_inventors, gg = None, []
         for name, games in GI.GAMES_BY_INVENTORS:
-            select_func = lambda gi, games=games: gi.id in games
-            if name is None or not filter(select_func, self.all_games_gi):
-               continue
+            def select_func(gi, games=games):
+                return gi.id in games
+            if name is None or not list(filter(
+                    select_func, self.all_games_gi)):
+                continue
             gg.append(SelectGameNode(None, name, select_func))
         if 1 and gg:
             s_by_inventors = SelectGameNode(None, _("by Inventors"),
                                             tuple(gg))
         #
-        ul_alternate_names = UserList(list(app.gdb.getGamesTuplesSortedByAlternateName()))
+        ul_alternate_names = UserList(
+            list(app.gdb.getGamesTuplesSortedByAlternateName()))
         #
-        self.rootnodes = filter(None, (
-            SelectGameNode(None, _("All Games"), None),
+        self.rootnodes = [_f for _f in (
+            SelectGameNode(None, _("All Games"), None, expanded=0),
             SelectGameNode(None, _("Alternate Names"), ul_alternate_names),
             SelectGameNode(None, _("Popular Games"),
                            lambda gi: gi.si.game_flags & GI.GT_POPULAR),
+            s_by_type,
             s_mahjongg,
             s_oriental,
             s_special,
             SelectGameNode(None, _("Custom Games"),
                            lambda gi: gi.si.game_type == GI.GT_CUSTOM),
-            s_by_type,
             SelectGameNode(None, _('by Skill Level'), (
                 SelectGameNode(None, _('Luck only'),
                                lambda gi: gi.skill_level == GI.SL_LUCK),
@@ -171,8 +182,9 @@ class SelectGameData(SelectDialogTreeData):
                                lambda gi: gi.skill_level == GI.SL_MOSTLY_LUCK),
                 SelectGameNode(None, _('Balanced'),
                                lambda gi: gi.skill_level == GI.SL_BALANCED),
-                SelectGameNode(None, _('Mostly skill'),
-                               lambda gi: gi.skill_level == GI.SL_MOSTLY_SKILL),
+                SelectGameNode(
+                    None, _('Mostly skill'),
+                    lambda gi: gi.skill_level == GI.SL_MOSTLY_SKILL),
                 SelectGameNode(None, _('Skill only'),
                                lambda gi: gi.skill_level == GI.SL_SKILL),
                 )),
@@ -192,8 +204,10 @@ class SelectGameData(SelectDialogTreeData):
                                    lambda gi: gi.si.ncards == 104),
                     SelectGameNode(None, _("144 cards"),
                                    lambda gi: gi.si.ncards == 144),
-                    SelectGameNode(None, _("Other number"),
-                                   lambda gi: gi.si.ncards not in (32, 48, 52, 64, 78, 104, 144)),
+                    SelectGameNode(
+                        None, _("Other number"),
+                        lambda gi: gi.si.ncards not in (32, 48, 52,
+                                                        64, 78, 104, 144)),
                 )),
                 SelectGameNode(None, _("by Number of Decks"), (
                     SelectGameNode(None, _("1 deck games"),
@@ -216,10 +230,11 @@ class SelectGameData(SelectDialogTreeData):
                                    lambda gi: gi.si.redeals == 3),
                     SelectGameNode(None, _("Unlimited redeals"),
                                    lambda gi: gi.si.redeals == -1),
-                    SelectGameNode(None, "Variable redeals",
+                    SelectGameNode(None, _("Variable redeals"),
                                    lambda gi: gi.si.redeals == -2),
-                    SelectGameNode(None, _("Other number of redeals"),
-                                   lambda gi: gi.si.redeals not in (-1, 0, 1, 2, 3)),
+                    SelectGameNode(
+                        None, _("Other number of redeals"),
+                        lambda gi: gi.si.redeals not in (-2, -1, 0, 1, 2, 3)),
                 )),
                 s_by_compatibility,
             )),
@@ -230,8 +245,14 @@ class SelectGameData(SelectDialogTreeData):
                                lambda gi: gi.si.game_flags & GI.GT_CHILDREN),
                 SelectGameNode(None, _("Games with Scoring"),
                                lambda gi: gi.si.game_flags & GI.GT_SCORE),
-                SelectGameNode(None, _("Games with Separate Decks"),
-                               lambda gi: gi.si.game_flags & GI.GT_SEPARATE_DECKS),
+                SelectGameNode(None, _("Games with Stripped Decks"),
+                               lambda gi: gi.si.game_flags & GI.GT_STRIPPED),
+                SelectGameNode(
+                    None, _("Games with Separate Decks"),
+                    lambda gi: gi.si.game_flags & GI.GT_SEPARATE_DECKS),
+                SelectGameNode(None, _("Games with Jokers"),
+                               lambda gi: gi.category == GI.GC_FRENCH and
+                               gi.subcategory == GI.GS_JOKER_DECK),
                 SelectGameNode(None, _("Open Games (all cards visible)"),
                                lambda gi: gi.si.game_flags & GI.GT_OPEN),
                 SelectGameNode(None, _("Relaxed Variants"),
@@ -239,7 +260,7 @@ class SelectGameData(SelectDialogTreeData):
             )),
             s_original,
             s_contrib,
-        ))
+        ) if _f]
 
 
 # ************************************************************************
@@ -290,8 +311,8 @@ class SelectGameDialog(MfxDialog):
     def initKw(self, kw):
         kw = KwStruct(kw,
                       strings=(None, None, _("&Cancel"),), default=0,
-                      separator=True,
                       resizable=True,
+                      separator=True,
                       padx=10, pady=10,
                       buttonpadx=10, buttonpady=5,
                       )
@@ -304,14 +325,15 @@ class SelectGameDialog(MfxDialog):
         MfxDialog.destroy(self)
 
     def mDone(self, button):
-        if button == 0:                    # Ok or double click
+        if button == 0:                 # Ok or double click
             self.gameid = self.tree.selection_key
             self.tree.n_expansions = 1  # save xyview in any case
-        if button == 1:                    # Rules
+        if button == 1:                # Rules
             doc = self.app.getGameRulesFilename(self.tree.selection_key)
             if not doc:
                 return
             dir = os.path.join("html", "rules")
+            from pysollib.help import help_html
             help_html(self.app, doc, dir, self.top)
             return
         MfxDialog.mDone(self, button)
@@ -347,16 +369,16 @@ class SelectGameDialogWithPreview(SelectGameDialog):
             w1, w2 = 220, 480
         else:
             w1, w2 = 200, 300
-        ##print sw, w1, w2
+        # print sw, w1, w2
         w2 = max(200, min(w2, 10 + 12*(app.subsampled_images.CARDW+10)))
-        ##print sw, w1, w2
-        ##padx, pady = kw.padx, kw.pady
-        padx, pady = kw.padx/2, kw.pady/2
+        # print sw, w1, w2
+        # padx, pady = kw.padx, kw.pady
+        padx, pady = kw.padx//2, kw.pady//2
         # PanedWindow
-        paned_window = Tkinter.PanedWindow(top_frame)
+        paned_window = tkinter.PanedWindow(top_frame)
         paned_window.pack(expand=True, fill='both')
-        left_frame = Tkinter.Frame(paned_window)
-        right_frame = Tkinter.Frame(paned_window)
+        left_frame = tkinter.Frame(paned_window)
+        right_frame = tkinter.Frame(paned_window)
         paned_window.add(left_frame)
         paned_window.add(right_frame)
         # Tree
@@ -365,15 +387,14 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                                     default=kw.default, font=font, width=w1)
         self.tree.frame.pack(padx=padx, pady=pady, expand=True, fill='both')
         # LabelFrame
-        info_frame = Tkinter.LabelFrame(right_frame, text=_('About game'))
-        stats_frame = Tkinter.LabelFrame(right_frame, text=_('Statistics'))
+        info_frame = tkinter.LabelFrame(right_frame, text=_('About game'))
+        stats_frame = tkinter.LabelFrame(right_frame, text=_('Statistics'))
         info_frame.grid(row=0, column=0, padx=padx, pady=pady,
                         ipadx=padx, ipady=pady, sticky='nws')
         stats_frame.grid(row=0, column=1, padx=padx, pady=pady,
                          ipadx=padx, ipady=pady, sticky='nws')
         # Info
         self.info_labels = {}
-        i = 0
         for n, t, f, row in (
             ('name',        _('Name:'),             info_frame,   0),
             ('altnames',    _('Alternate names:'),  info_frame,   1),
@@ -389,13 +410,13 @@ class SelectGameDialogWithPreview(SelectGameDialog):
             ('time',        _('Playing time:'),     stats_frame,  3),
             ('moves',       _('Moves:'),            stats_frame,  4),
             ('percent',     _('% won:'),            stats_frame,  5),
-            ):
-            title_label = Tkinter.Label(f, text=t, justify='left', anchor='w')
+                ):
+            title_label = tkinter.Label(f, text=t, justify='left', anchor='w')
             title_label.grid(row=row, column=0, sticky='nw')
-            text_label = Tkinter.Label(f, justify='left', anchor='w')
+            text_label = tkinter.Label(f, justify='left', anchor='w')
             text_label.grid(row=row, column=1, sticky='nw')
             self.info_labels[n] = (title_label, text_label)
-        ##info_frame.columnconfigure(1, weight=1)
+        # info_frame.columnconfigure(1, weight=1)
         info_frame.rowconfigure(6, weight=1)
         stats_frame.rowconfigure(6, weight=1)
         # Canvas
@@ -414,12 +435,13 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         self.preview_game = None
         self.preview_app = None
         self.updatePreview(gameid, animations=0)
-        ##focus = self.tree.frame
+        # focus = self.tree.frame
         self.mainloop(focus, kw.timeout)
 
     def initKw(self, kw):
         kw = KwStruct(kw,
-                      strings=(_("&Select"), _("&Rules"), _("&Cancel"),),
+                      strings=((_("&Rules"), 10), 'sep',
+                               _("&Select"), _("&Cancel"),),
                       default=0,
                       )
         return SelectGameDialog.initKw(self, kw)
@@ -438,7 +460,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
             if destroy:
                 self.preview.canvas.delete("all")
         #
-        #for l in self.info_labels.values():
+        # for l in self.info_labels.values():
         #    l.config(text='')
         # destruct the game
         if self.preview_game:
@@ -466,39 +488,61 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         if self.preview_app is None:
             self.preview_app = Struct(
                 # variables
-                audio = self.app.audio,
-                canvas = canvas,
-                cardset = self.app.cardset.copy(),
-                comments = self.app.comments.new(),
-                gamerandom = self.app.gamerandom,
-                gdb = self.app.gdb,
-                gimages = self.app.gimages,
-                images = self.app.subsampled_images,
-                menubar = None,
-                miscrandom = self.app.miscrandom,
-                opt = self.app.opt.copy(),
-                startup_opt = self.app.startup_opt,
-                stats = self.app.stats.new(),
-                top = None,
-                top_cursor = self.app.top_cursor,
-                toolbar = None,
+                audio=self.app.audio,
+                canvas=canvas,
+                cardset=self.app.cardset.copy(),
+                gamerandom=self.app.gamerandom,
+                gdb=self.app.gdb,
+                gimages=self.app.gimages,
+                images=None,
+                menubar=None,
+                miscrandom=self.app.miscrandom,
+                opt=self.app.opt.copy(),
+                startup_opt=self.app.startup_opt,
+                stats=self.app.stats.new(),
+                top=None,
+                top_cursor=self.app.top_cursor,
+                toolbar=None,
                 # methods
-                constructGame = self.app.constructGame,
-                getFont = self.app.getFont,
+                constructGame=self.app.constructGame,
+                getFont=self.app.getFont,
             )
             self.preview_app.opt.shadow = 0
             self.preview_app.opt.shade = 0
         #
-        self.preview_app.audio = None    # turn off audio for intial dealing
+
+        c = self.app.cardsets_cache.get(gi.category)
+        c2 = None
+        if c:
+            c2 = c.get(gi.subcategory)
+        if not c2:
+            cardset = self.app.cardset_manager.getByName(
+                self.app.opt.cardset[gi.category][gi.subcategory][0])
+            self.app.loadCardset(cardset, id=gi.category,
+                                 tocache=True, noprogress=True)
+            c = self.app.cardsets_cache.get(gi.category)
+            if c:
+                c2 = c.get(gi.subcategory)
+            if not c2:
+                c = self.app.cardsets_cache.get(cardset.type)
+                if c:
+                    c2 = c.get(cardset.subtype)
+        if c2:
+            self.preview_app.images = c2[2]
+        else:
+            self.preview_app.images = self.app.subsampled_images
+
+        self.preview_app.audio = None    # turn off audio for initial dealing
         if animations >= 0:
             self.preview_app.opt.animations = animations
         #
         if self.preview_game:
             self.preview_game.endGame()
             self.preview_game.destruct()
-        ##self.top.wm_title("Select Game - " + self.app.getGameTitleName(gameid))
+        # self.top.wm_title("Select Game - " +
+        #   self.app.getGameTitleName(gameid))
         title = self.app.getGameTitleName(gameid)
-        self.top.wm_title(_("Playable Preview - ") + title)
+        self.top.wm_title(_("Select Game - %(game)s") % {'game': title})
         #
         self.preview_game = gi.gameclass(gi)
         self.preview_game.createPreview(self.preview_app)
@@ -527,7 +571,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         #
         self.updateInfo(gameid)
         #
-        rules_button = self.buttons[1]
+        rules_button = self.buttons[0]
         if self.app.getGameRulesFilename(gameid):
             rules_button.config(state="normal")
         else:
@@ -542,21 +586,20 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         type = ''
         if gi.si.game_type in GI.TYPE_NAMES:
             type = _(GI.TYPE_NAMES[gi.si.game_type])
-        sl = {
-            GI.SL_LUCK:         _('Luck only'),
-            GI.SL_MOSTLY_LUCK:  _('Mostly luck'),
-            GI.SL_BALANCED:     _('Balanced'),
-            GI.SL_MOSTLY_SKILL: _('Mostly skill'),
-            GI.SL_SKILL:        _('Skill only'),
-            }
-        skill_level = sl.get(gi.skill_level)
-        if    gi.redeals == -2: redeals = _('variable')
-        elif  gi.redeals == -1: redeals = _('unlimited')
-        else:                   redeals = str(gi.redeals)
+        skill_level = GI.SKILL_LEVELS.get(gi.skill_level)
+        if gi.redeals == -2:
+            redeals = _('variable')
+        elif gi.redeals == -1:
+            redeals = _('unlimited')
+        else:
+            redeals = str(gi.redeals)
         # stats
-        won, lost, time, moves = self.app.stats.getFullStats(self.app.opt.player, gameid)
-        if won+lost > 0: percent = "%.1f" % (100.0*won/(won+lost))
-        else: percent = "0.0"
+        won, lost, time, moves = self.app.stats.getFullStats(
+            self.app.opt.player, gameid)
+        if won+lost > 0:
+            percent = "%.1f" % (100.0*won/(won+lost))
+        else:
+            percent = "0.0"
         time = format_time(time)
         moves = str(round(moves, 1))
         for n, t in (
@@ -573,7 +616,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
             ('time',        time),
             ('moves',       moves),
             ('percent',     percent),
-            ):
+                ):
             title_label, text_label = self.info_labels[n]
             if t in ('', None):
                 title_label.grid_remove()
@@ -582,4 +625,3 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                 title_label.grid()
                 text_label.grid()
             text_label.config(text=t)
-
